IncludeScript("vsoundscaper/soundscape_config.nut")

::soundscapes <- {}

::FADE_TYPES <- {
    FADE_OUT = -1,
    FADE_NONE = 0,
    FADE_IN = 1
}

function OnPostSpawn()
{
    //reset songs between rounds
    for (local i = 1; i <= MaxClients(); i++){
        local player = PlayerInstanceFromIndex(i)
        if(player){
            foreach (song in all_songs){
                EmitSoundEx({
                    sound_name = song
                    entity = player
                    filter_type = 4
                    flags = 4
                    channel = 6
                })
            }
        }
    }
}

//defined in the soundscape_config.nut file!
function InitializeSoundscape(playlist_name)
{
    if (playlist_name in soundscapes) return;
    soundscapes[playlist_name] <- {}
    soundscapes[playlist_name].playlist <- playlists[playlist_name]
    local sound_ent = Entities.CreateByClassname("logic_relay")
    sound_ent.ValidateScriptScope()
    local scope = sound_ent.GetScriptScope()
    soundscapes[playlist_name].ent <- sound_ent

    scope.scripter <- this
    scope.playlist <- playlists[playlist_name]
    scope.playlist_name <- playlist_name
    scope.now_playing <- -1
    scope.start_time <- 0
    scope.current_time <- 0
    scope.player_list <- {}
    scope.SoundscapeThink <- SoundscapeThink
    AddThinkToEnt(sound_ent, "SoundscapeThink")
    return
}

function SoundscapeThink()
{
    current_time = Time()

    if (now_playing == -1)
    {
        local song = RandomInt(0,playlist.len()-1)
        start_time = Time()
        current_time = start_time
        now_playing = song
    }

    foreach (player,values in player_list)
    {
        local new_volume = values.volume + (values.fade_type * 0.10)
        values.volume = new_volume
        if (new_volume > 0)
        {
            if ( new_volume >= 1.0 && values.fade_type == FADE_TYPES.FADE_IN)
            {
                values.volume = 1.0
                values.fade_type = FADE_TYPES.FADE_NONE
            }
            EmitSoundEx({
                sound_name = playlist[now_playing][0]
                volume = values.volume
                entity = player
                filter_type = 4
                flags = 1
                channel = 6
            })
        }
        else
        {
            EmitSoundEx({
                sound_name = playlist[now_playing][0]
                entity = player
                filter_type = 4
                flags = 4
                channel = 6
            })
            delete player_list[player]
            if (player_list.len() == 0)
            {
                soundscapes[playlist_name].ent.Kill()
                delete soundscapes[playlist_name]
            }
        }
    }

    if (current_time - start_time > playlist[now_playing][1])
    {
        now_playing = -1
    }

    return 0.10
}

function AddPlayerToSoundscape(playlist_name,player)
{
    if (!(playlist_name in soundscapes))
        InitializeSoundscape(playlist_name)
    local ss = soundscapes[playlist_name].ent.GetScriptScope()
    if (player in ss.player_list) {
        if (ss.player_list[player].fade_type == FADE_TYPES.FADE_OUT)
        {
            ss.player_list[player].fade_type = FADE_TYPES.FADE_IN
        }
    }
    else {
        ss.player_list[player] <- {
            volume = 0,
            fade_type = FADE_TYPES.FADE_IN
        }
    }
}

function RemovePlayerFromSoundscape(playlist_name,player)
{
    if (!(playlist_name in soundscapes)) return
    local ss = soundscapes[playlist_name].ent.GetScriptScope()
    if (player in ss.player_list) {
        ss.player_list[player].fade_type = FADE_TYPES.FADE_OUT
    }
}